@@ -4,7 +4,7 @@
  * Created Date: 24/05/2021
  * Author: Shun Suzuki
  * -----
- * Last Modified: 06/07/2021
+ * Last Modified: 16/12/2021
  * Modified By: Shun Suzuki (suzuki@hapis.k.u-tokyo.ac.jp)
  * -----
  * Copyright (c) 2021 Hapis Lab. All rights reserved.
@@ -21,15 +21,60 @@ use crate::hardware_defined::is_missing_transducer;
 
 use super::hardware_defined::{NUM_TRANS_IN_UNIT, NUM_TRANS_X, NUM_TRANS_Y, TRANS_SPACING_MM};
 
-struct Device {
-    global_trans_positions: Vec<Vector3>,
+pub struct Transducer {
+    id: usize,
+    pos: Vector3,
     x_direction: Vector3,
     y_direction: Vector3,
     z_direction: Vector3,
 }
 
+impl Transducer {
+    pub fn new(
+        id: usize,
+        x: f64,
+        y: f64,
+        z: f64,
+        x_direction: Vector3,
+        y_direction: Vector3,
+        z_direction: Vector3,
+    ) -> Self {
+        Self {
+            id,
+            pos: Vector3::new(x, y, z),
+            x_direction,
+            y_direction,
+            z_direction,
+        }
+    }
+
+    pub fn id(&self) -> usize {
+        self.id
+    }
+
+    pub fn position(&self) -> &Vector3 {
+        &self.pos
+    }
+
+    pub fn x_direction(&self) -> &Vector3 {
+        &self.x_direction
+    }
+
+    pub fn y_direction(&self) -> &Vector3 {
+        &self.y_direction
+    }
+
+    pub fn z_direction(&self) -> &Vector3 {
+        &self.z_direction
+    }
+}
+
+pub struct Device {
+    global_trans_positions: Vec<Transducer>,
+}
+
 impl Device {
-    pub fn new(position: Vector3, rotation: UnitQuaternion) -> Device {
+    pub fn new(id: usize, position: Vector3, rotation: UnitQuaternion) -> Self {
         let rot_mat: Matrix4x4 = From::from(rotation);
         let trans_mat = rot_mat.append_translation(&position);
         let x_direction = Self::get_direction(Vector3::x(), rotation);
@@ -37,25 +82,34 @@ impl Device {
         let z_direction = Self::get_direction(Vector3::z(), rotation);
 
         let mut global_trans_positions = Vec::with_capacity(NUM_TRANS_IN_UNIT);
+        let mut i = id * NUM_TRANS_IN_UNIT;
         for y in 0..NUM_TRANS_Y {
             for x in 0..NUM_TRANS_X {
-                if !is_missing_transducer(x, y) {
-                    let local_pos = Vector4::new(
-                        x as f64 * TRANS_SPACING_MM,
-                        y as f64 * TRANS_SPACING_MM,
-                        0.,
-                        1.,
-                    );
-                    global_trans_positions.push(convert_to_vec3(trans_mat * local_pos));
+                if is_missing_transducer(x, y) {
+                    continue;
                 }
+                let local_pos = Vector4::new(
+                    x as f64 * TRANS_SPACING_MM,
+                    y as f64 * TRANS_SPACING_MM,
+                    0.,
+                    1.,
+                );
+                let homo = trans_mat * local_pos;
+                global_trans_positions.push(Transducer::new(
+                    i,
+                    homo.x,
+                    homo.y,
+                    homo.z,
+                    x_direction,
+                    y_direction,
+                    z_direction,
+                ));
+                i += 1;
             }
         }
 
         Device {
             global_trans_positions,
-            x_direction,
-            y_direction,
-            z_direction,
         }
     }
 
@@ -63,6 +117,19 @@ impl Device {
         let dir: UnitQuaternion = UnitQuaternion::from_quaternion(Quaternion::from_imag(dir));
         (rotation * dir * rotation.conjugate()).imag().normalize()
     }
+
+    pub fn local_position(&self, global_position: Vector3) -> Vector3 {
+        let local_origin = self.global_trans_positions[0].position();
+        let x_dir = self.global_trans_positions[0].x_direction();
+        let y_dir = self.global_trans_positions[0].y_direction();
+        let z_dir = self.global_trans_positions[0].z_direction();
+        let rv = global_position - local_origin;
+        Vector3::new(rv.dot(x_dir), rv.dot(y_dir), rv.dot(z_dir))
+    }
+
+    pub fn transducers(&self) -> &[Transducer] {
+        &self.global_trans_positions
+    }
 }
 
 #[derive(Default)]
@@ -120,50 +187,19 @@ impl Geometry {
     /// * `rot` - Rotation quaternion.
     ///
     pub fn add_device_quaternion(&mut self, position: Vector3, rotation: UnitQuaternion) {
-        self.devices.push(Device::new(position, rotation));
+        let id = self.devices.len();
+        self.devices.push(Device::new(id, position, rotation));
     }
 
     pub fn num_devices(&self) -> usize {
         self.devices.len()
     }
 
-    pub fn position_by_global_idx(&self, global_transducer_idx: usize) -> Vector3 {
-        let local_trans_idx = global_transducer_idx % NUM_TRANS_IN_UNIT;
-        let device_idx = global_transducer_idx / NUM_TRANS_IN_UNIT;
-        self.position_by_local_idx(device_idx, local_trans_idx)
+    pub fn devices(&self) -> &[Device] {
+        &self.devices
     }
 
-    pub fn position_by_local_idx(&self, device_idx: usize, local_trans_idx: usize) -> Vector3 {
-        self.devices[device_idx].global_trans_positions[local_trans_idx]
-    }
-
-    pub fn local_position(&self, device_idx: usize, global_position: Vector3) -> Vector3 {
-        let device = &self.devices[device_idx];
-        let local_origin = device.global_trans_positions[0];
-        let x_dir = device.x_direction;
-        let y_dir = device.y_direction;
-        let z_dir = device.z_direction;
-        let rv = global_position - local_origin;
-        Vector3::new(rv.dot(&x_dir), rv.dot(&y_dir), rv.dot(&z_dir))
-    }
-
-    pub fn direction(&self, device_idx: usize) -> Vector3 {
-        self.devices[device_idx].z_direction
-    }
-
-    pub fn x_direction(&self, device_idx: usize) -> Vector3 {
-        self.devices[device_idx].x_direction
-    }
-
-    pub fn y_direction(&self, device_idx: usize) -> Vector3 {
-        self.devices[device_idx].y_direction
-    }
-
-    pub fn z_direction(&self, device_idx: usize) -> Vector3 {
-        self.devices[device_idx].z_direction
+    pub fn transducers(&self) -> impl Iterator<Item = &Transducer> {
+        self.devices.iter().flat_map(|dev| dev.transducers())
     }
 }
-
-fn convert_to_vec3(v: Vector4) -> Vector3 {
-    Vector3::new(v.x, v.y, v.z)
-}
