@@ -4,7 +4,7 @@
  * Created Date: 24/05/2021
  * Author: Shun Suzuki
  * -----
- * Last Modified: 26/05/2021
+ * Last Modified: 16/12/2021
  * Modified By: Shun Suzuki (suzuki@hapis.k.u-tokyo.ac.jp)
  * -----
  * Copyright (c) 2021 Hapis Lab. All rights reserved.
@@ -13,11 +13,13 @@
 
 use anyhow::Result;
 
+use crate::hardware_defined::{RxDatagram, TxDatagram};
+
 /// Link is a interface to the AUTD device.
 pub trait Link: Send {
     fn open(&mut self) -> Result<()>;
     fn close(&mut self) -> Result<()>;
-    fn send(&mut self, data: &[u8]) -> Result<bool>;
-    fn read(&mut self, data: &mut [u8]) -> Result<bool>;
+    fn send(&mut self, tx: &TxDatagram) -> Result<bool>;
+    fn receive(&mut self, rx: &mut RxDatagram) -> Result<bool>;
     fn is_open(&self) -> bool;
 }
